@@ -34,6 +34,7 @@
 #include "core.h"
 #include "gene-algorithms.h"
 #include "sambam-file.h"
+#include "input-files.h"
 
 int SamBam_fetch_next_chunk(SamBam_FILE *fp)
 {
@@ -151,7 +152,7 @@ SamBam_FILE * SamBam_fopen(char * fname , int file_type)
 		if(first_ch!=31 || second_ch!=139)
 		{
 			free(ret);
-			SUBREADprintf("Not a BAM file: %d %d\n", first_ch, second_ch);
+			SUBREADprintf("Not a BAM file! %d %d\n", first_ch, second_ch);
 			return NULL;
 		}
 
@@ -1089,14 +1090,14 @@ int test_bamview(int argc, char ** argv)
 	return 0;
 }
 
-int SamBam_writer_create(SamBam_Writer * writer, char * BAM_fname, int threads, int keep_in_memory, char * tmpfname)
+int SamBam_writer_create(SamBam_Writer * writer, char * BAM_fname, int threads, int sort_reads_by_coord, char * tmpfname)
 {
 	memset(writer, 0, sizeof(SamBam_Writer));
 
 	if(BAM_fname)
 	{
 		writer -> bam_fp = f_subr_open(BAM_fname, "wb");
-		if(keep_in_memory){
+		if(sort_reads_by_coord){
 			char tname[MAX_FILE_NAME_LENGTH];
 			sprintf(tname, "%s.bai", BAM_fname);
 			writer -> BAI_fp = f_subr_open(tname, "wb");
@@ -1109,7 +1110,8 @@ int SamBam_writer_create(SamBam_Writer * writer, char * BAM_fname, int threads,
 	#endif
 
 	writer -> threads = threads;
-	writer -> keep_in_memory = keep_in_memory;
+	writer -> sort_reads_by_coord = sort_reads_by_coord;
+	writer -> fastest_compression = 0;
 	writer -> compressed_chunk_buffer = malloc(70000); 
 	writer -> chunk_buffer = malloc(70000); 
 	writer -> chunk_buffer_max_size = 70000;
@@ -1203,7 +1205,7 @@ void SamBam_writer_add_chunk(SamBam_Writer * writer, int thread_no)
 	this_stream -> zfree = Z_NULL;
 	this_stream -> opaque = Z_NULL;
 
-	deflateInit2(this_stream, SAMBAM_COMPRESS_LEVEL, Z_DEFLATED,
+	deflateInit2(this_stream, writer -> fastest_compression? SAMBAM_COMPRESS_LEVEL_FASTEST : SAMBAM_COMPRESS_LEVEL_NORMAL, Z_DEFLATED,
 		SAMBAM_GZIP_WINDOW_BITS, Z_DEFAULT_MEM_LEVEL, Z_DEFAULT_STRATEGY);
 	
 	this_stream -> next_in = (unsigned char *)this_buffer;
@@ -1303,7 +1305,7 @@ int SamBam_writer_close(SamBam_Writer * writer)
 			SamBam_writer_write_header(writer);
 	} else SamBam_writer_finalise_one_thread(writer);
 
-	if(writer -> keep_in_memory)
+	if(writer -> sort_reads_by_coord)
 		SamBam_writer_sort_bins_to_BAM(writer);
 	
 	writer -> chunk_buffer_used = 0;
@@ -1356,6 +1358,8 @@ int SamBam_writer_add_header(SamBam_Writer * writer, char * header_text, int add
 	writer -> header_plain_text_buffer_used += new_text_len;
 	strcpy(writer -> header_plain_text_buffer + writer -> header_plain_text_buffer_used, "\n");
 	writer -> header_plain_text_buffer_used ++;
+
+//	SUBREADprintf("ADJHEADERXCD BIN=%s\n", header_text);
 	if(add_chro && memcmp(header_text, "@SQ",3)==0)
 	{
 		char * chro = NULL;
@@ -1386,6 +1390,7 @@ int SamBam_writer_add_chromosome(SamBam_Writer * writer, char * chro_name, unsig
 {
 	unsigned int chro_id = writer -> chromosome_name_table -> numOfElements;
 
+	//SUBREADprintf("ADJHEADER_CHRO %s of %u\n", chro_name, chro_length);
 	//assert(strlen(chro_name) < 30);
 
 	char * chro_name_space = malloc(strlen(chro_name)+1);
@@ -1618,6 +1623,97 @@ char * duplicate_TAB_record_field(char * rline, int fno, int to_end){
 	return ret;
 }
 
+int SamBam_writer_add_read_fqs_scRNA(gzFile *outfps, char * bambin){
+	int reclen=0;
+	gzFile outR1fp = outfps[0];
+	gzFile outI1fp = outfps[1];
+	gzFile outR2fp = outfps[2];
+
+	memcpy(&reclen, bambin,4);
+	int flag = 0, l_seq = 0, l_read_name = 0, n_cigar_ops = 0;
+	memcpy(&l_read_name, bambin+12,1);
+	memcpy(&n_cigar_ops, bambin+16,2);
+	memcpy(&flag, bambin+18,2);
+	memcpy(&l_seq, bambin+20,4);
+
+	int x1=0;
+
+	gzwrite(outR2fp,"@",1);
+	gzwrite(outR1fp,"@",1);
+	gzwrite(outI1fp,"@",1);
+	char * readname = bambin+36;
+	gzwrite(outR1fp,readname, 12);
+	gzwrite(outR2fp,readname, 12);
+	gzwrite(outI1fp,readname, 12);
+	gzwrite(outR1fp,"\n",1);
+	gzwrite(outR2fp,"\n",1);
+	gzwrite(outI1fp,"\n",1);
+
+	char * R1seq = bambin+36+13;
+	int R1len = 0;
+	for(R1len=0; R1seq[R1len] && R1seq[R1len]!='|' ;R1len++);
+	char * R1qual = R1seq + R1len + 1;
+	gzwrite(outR1fp,R1seq, R1len);
+	gzwrite(outR1fp,"\n+\n",3);
+	gzwrite(outR1fp,R1qual, R1len);
+	gzwrite(outR1fp,"\n",1);
+
+	char * I1seq = R1qual + R1len + 1;
+	int I1len = 0;
+	for(I1len=0; I1seq[I1len] && I1seq[I1len]!='|' ;I1len++);
+	char * I1qual = I1seq + I1len + 1;
+	gzwrite(outI1fp,I1seq, I1len);
+	gzwrite(outI1fp,"\n+\n",3);
+	gzwrite(outI1fp,I1qual, I1len);
+	gzwrite(outI1fp,"\n",1);
+
+	char oseq[l_seq+1];
+	int seqbase = 36+l_read_name+n_cigar_ops*4;
+	for(x1=0; x1<l_seq;x1++)oseq[x1]="=ACMGRSVTWYHKDBN"[(bambin[ seqbase+(x1/2)] >> (((x1%2)?0:1) *4) )&0xf];
+	oseq[l_seq]=0;
+	if(flag & 16) reverse_read(oseq,l_seq, GENE_SPACE_BASE);
+	gzwrite(outR2fp, oseq, l_seq);
+	gzwrite(outR2fp,"\n+\n",3);
+
+	seqbase = 36+l_read_name+n_cigar_ops*4 + ( l_seq+1 )/2;
+	for(x1=0; x1<l_seq;x1++)oseq[x1]=33+bambin[ seqbase+x1];
+	if(flag & 16)reverse_quality(oseq, l_seq);
+	oseq[l_seq]=0;
+	gzwrite(outR2fp, oseq, l_seq);
+	gzwrite(outR2fp,"\n",1);
+	return 0;
+}
+
+int SamBam_writer_add_read_bin(SamBam_Writer * writer, int thread_no, char * rbin, int committable){
+	char * this_chunk_buffer=NULL;
+	srInt_64 *this_chunk_buffer_used=NULL;
+	if(thread_no >= 0){
+		if(writer -> sort_reads_by_coord && writer -> threads_chunk_buffer_max_size[thread_no] < writer -> threads_chunk_buffer_used[thread_no] + 12000){
+			writer -> threads_chunk_buffer_max_size[thread_no] = writer -> threads_chunk_buffer_max_size[thread_no] * 7/4;
+			writer -> threads_chunk_buffer[ thread_no ] = realloc(writer -> threads_chunk_buffer[ thread_no ], writer -> threads_chunk_buffer_max_size[thread_no]);
+		}
+		this_chunk_buffer = writer -> threads_chunk_buffer[ thread_no ];
+		this_chunk_buffer_used = writer -> threads_chunk_buffer_used + thread_no;
+	}else{
+		if(writer -> sort_reads_by_coord && writer -> chunk_buffer_max_size < writer -> chunk_buffer_used + 12000){
+			//SUBREADprintf("REALLOCATE MEM : %d -> %d\n", writer -> chunk_buffer_max_size,  writer -> chunk_buffer_max_size * 7/4);
+			writer -> chunk_buffer_max_size = writer -> chunk_buffer_max_size * 7/4;
+			writer -> chunk_buffer = realloc(writer -> chunk_buffer, writer -> chunk_buffer_max_size);
+		}
+		this_chunk_buffer = writer -> chunk_buffer;
+		this_chunk_buffer_used = &writer -> chunk_buffer_used;
+	}
+	//SUBREADprintf("WRTXBIN: PTR %p of USED %lld at read %p\n", this_chunk_buffer, *this_chunk_buffer_used, rbin);
+	int reclen=0;
+	memcpy(&reclen, rbin,4);
+	memcpy(this_chunk_buffer+(*this_chunk_buffer_used), rbin, reclen+4);
+	(*this_chunk_buffer_used)+= reclen+4;
+
+	if((*this_chunk_buffer_used)>55000 && committable && !writer -> sort_reads_by_coord)
+		SamBam_writer_add_chunk(writer, thread_no);
+	//SUBREADprintf("WRTXBIN: FIN WTR %p of USED %lld\n", this_chunk_buffer, *this_chunk_buffer_used);
+	return 0;
+}
 int SamBam_writer_add_read_line(SamBam_Writer * writer, int thread_no, char * rline, int committable){
 	char * read_name, * flag_str, *chro_name, *chro_position_str, * mapping_quality_str, * cigar, * next_chro_name, *next_chro_position_str, *temp_len_str, *read_text, *qual_text, *additional_columns;
 	read_name = duplicate_TAB_record_field(rline, 0,0);
@@ -1673,17 +1769,17 @@ int SamBam_writer_add_read(SamBam_Writer * writer, int thread_no, char * read_na
 
 
 	char * this_chunk_buffer;
-	long long * this_chunk_buffer_used;
+	srInt_64 * this_chunk_buffer_used;
 
 	if(thread_no >= 0){
-		if(writer -> keep_in_memory && writer -> threads_chunk_buffer_max_size[thread_no] < writer -> threads_chunk_buffer_used[thread_no] + 12000){
+		if(writer -> sort_reads_by_coord && writer -> threads_chunk_buffer_max_size[thread_no] < writer -> threads_chunk_buffer_used[thread_no] + 12000){
 			writer -> threads_chunk_buffer_max_size[thread_no] = writer -> threads_chunk_buffer_max_size[thread_no] * 7/4;
 			writer -> threads_chunk_buffer[ thread_no ] = realloc(writer -> threads_chunk_buffer[ thread_no ], writer -> threads_chunk_buffer_max_size[thread_no]);
 		}
 		this_chunk_buffer = writer -> threads_chunk_buffer[ thread_no ];
 		this_chunk_buffer_used = writer -> threads_chunk_buffer_used + thread_no;
 	}else{
-		if(writer -> keep_in_memory && writer -> chunk_buffer_max_size < writer -> chunk_buffer_used + 12000){
+		if(writer -> sort_reads_by_coord && writer -> chunk_buffer_max_size < writer -> chunk_buffer_used + 12000){
 			//SUBREADprintf("REALLOCATE MEM : %d -> %d\n", writer -> chunk_buffer_max_size,  writer -> chunk_buffer_max_size * 7/4);
 			writer -> chunk_buffer_max_size = writer -> chunk_buffer_max_size * 7/4;
 			writer -> chunk_buffer = realloc(writer -> chunk_buffer, writer -> chunk_buffer_max_size);
@@ -1743,7 +1839,7 @@ int SamBam_writer_add_read(SamBam_Writer * writer, int thread_no, char * read_na
 	(*this_chunk_buffer_used) += additional_bin_len;
 
 
-	if((*this_chunk_buffer_used)>55000 && committable && !writer -> keep_in_memory)
+	if((*this_chunk_buffer_used)>55000 && committable && !writer -> sort_reads_by_coord)
 		SamBam_writer_add_chunk(writer, thread_no);
 
 	return 0;
@@ -1997,7 +2093,7 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 				current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
 				sprintf(tfpx , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii + merge_i);
-				sb_fps[bii] = fopen(tfpx,"r");
+				sb_fps[bii] = fopen(tfpx,"rb");
 				current_min_fps[bii] = SamBam_writer_sort_bins_to_BAM_FP_pos(sb_fps[bii]);
 				if(current_min_fps[bii] < SUBREAD_MAX_ULONGLONG && current_min_fps[bii] < current_min){
 					current_min = current_min_fps[bii];
@@ -2183,7 +2279,6 @@ void SamBam_writer_optimize_bins(HashTable *bin_tab, ArrayList *bin_arr, HashTab
 
 // MUST be called by THREAD 0!
 void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
-	print_in_box(80,0,0,"Sort the reads...");
 	SamBam_writer_one_thread_merge_sortedbins(writer);
 
 	FILE ** sb_fps = malloc(sizeof(FILE *) * writer -> sorted_batch_id);
@@ -2198,7 +2293,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 		current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
 		sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
-		sb_fps[bii] = fopen(tfp,"r");
+		sb_fps[bii] = fopen(tfp,"rb");
 		if(sb_fps[bii]!=NULL){
 			current_min_fps[bii] = SamBam_writer_sort_bins_to_BAM_FP_pos(sb_fps[bii]);
 			if(current_min_fps[bii] < SUBREAD_MAX_ULONGLONG && current_min_fps[bii] < current_min){
@@ -2305,7 +2400,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 }
 
 void SamBam_writer_finalise_thread(SamBam_Writer * writer, int thread_id){
-	if(writer -> keep_in_memory){
+	if(writer -> sort_reads_by_coord){
 		SamBam_writer_sort_buff_one_write(writer, writer -> threads_chunk_buffer[thread_id], writer -> threads_chunk_buffer_used[thread_id], thread_id);
 		writer -> threads_chunk_buffer_used [thread_id] = 0;
 	}else{
@@ -2315,7 +2410,7 @@ void SamBam_writer_finalise_thread(SamBam_Writer * writer, int thread_id){
 
 void SamBam_writer_finalise_one_thread(SamBam_Writer * writer){
 	if(writer -> threads < 2){
-		if(writer -> keep_in_memory){
+		if(writer -> sort_reads_by_coord){
 			if(writer -> chunk_buffer_used>0){
 				SamBam_writer_sort_buff_one_write(writer, writer -> chunk_buffer, writer -> chunk_buffer_used, -1);
 				writer -> chunk_buffer_used = 0;
